.parler-language-tabs span {
    display: inline-block;
    padding: 5px 15px;
    border: 1px solid #ccc;
    border-bottom: none;
    position: relative;
    left: 0;
    top: 1px;
    font-weight: bold;
}
.parler-language-tabs span.current {
    border-bottom: 1px solid #fff;
}
.parler-language-tabs span.empty {
    opacity: 0.7;
    font-weight: normal;
}
.parler-language-tabs a.deletelink {
    right: -17px;
    bottom: 4px;
    position: relative;
}


.parler-inline-language-tabs + .inline-group {
    margin-top: 0;
}